"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.padToNBytes = padToNBytes;
exports.getZeroOffsetArrayBuffer = getZeroOffsetArrayBuffer;
exports.copyArrayBuffer = copyArrayBuffer;
exports.copyToArray = copyToArray;

var _assert = _interopRequireDefault(require("../env-utils/assert"));

var _arrayBufferUtils = require("./array-buffer-utils");

function padToNBytes(byteLength, padding) {
  (0, _assert["default"])(byteLength >= 0);
  (0, _assert["default"])(padding > 0);
  return byteLength + (padding - 1) & ~(padding - 1);
}

function getZeroOffsetArrayBuffer(arrayBuffer, byteOffset, byteLength) {
  return (0, _arrayBufferUtils.sliceArrayBuffer)(arrayBuffer, byteOffset, byteLength);
}

function copyArrayBuffer(targetBuffer, sourceBuffer, byteOffset) {
  var byteLength = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : sourceBuffer.byteLength;
  var targetArray = new Uint8Array(targetBuffer, byteOffset, byteLength);
  var sourceArray = new Uint8Array(sourceBuffer);
  targetArray.set(sourceArray);
  return targetBuffer;
}

function copyToArray(source, target, targetOffset) {
  var sourceArray;

  if (source instanceof ArrayBuffer) {
    sourceArray = new Uint8Array(source);
  } else {
    var srcByteOffset = source.byteOffset;
    var srcByteLength = source.byteLength;
    sourceArray = new Uint8Array(source.buffer || source.arrayBuffer, srcByteOffset, srcByteLength);
  }

  target.set(sourceArray, targetOffset);
  return targetOffset + padToNBytes(sourceArray.byteLength, 4);
}
//# sourceMappingURL=memory-copy-utils.js.map